package com.ether.dao;

import com.ether.Data;
import com.ether.PersistenceException;
import com.medias.Nombre;
import com.medias.database.objects.Valeur;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author vmipsl
 * @date 07 apr 2011
 */
public class ValueDAOImpl
        extends DomainAccessObjectImpl<Valeur, Integer>
        implements ValueDAO
{
    protected ValueDAOImpl()
    {
        super( Valeur.class );
    }

    @NotNull
    public <T1, T2, T3> Data<T1[], T2[], T3[]> getListsByPlateformByParameterByPeriodForTimeSerie( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( Valeur.class, "value" )
                .add( Restrictions.eq( "value.parametre.id", parameterId ) )
                .createCriteria( "mesure", "measure" )
                .add( Restrictions.eq( "measure.plateforme.id", plateformId ) );

        if( null != dateBegin )
            criteria.add( Restrictions.ge( "measure.mesureDate", dateBegin ) );
        if( null != dateEnd )
            criteria.add( Restrictions.le( "measure.mesureDate", dateEnd ) );

        criteria.setProjection( Projections.distinct( Projections.projectionList()
                .add( Projections.property( "value.valeurVal" ) )
                .add( Projections.property( "measure.mesureDate" ) ) ) );
        criteria.addOrder( Order.asc( "measure.mesureDate" ) );

        final List<Object[]> objects = selectAllByCriteria( Object[].class, criteria );

        final double[] values = new double[objects.size()];
        final Date[] dates = new Date[objects.size()];

        int i = 0;
        for( final Object[] value : objects )
        {
            values[i] = (Double) value[0];
            dates[i] = (Date) value[1];
            i++;
        }

        return new Data( values, dates );
    }

    @NotNull
    public <T1, T2, T3> Data<T1[], T2[], T3[]> getArraysByPlateformByParameterByPeriodFor2D( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( Valeur.class, "value" )
                .add( Restrictions.eq( "value.parametre.id", parameterId ) )
                .createCriteria( "mesure", "measure" )
                .add( Restrictions.eq( "measure.plateforme.id", plateformId ) );

        if( null != dateBegin )
            criteria.add( Restrictions.ge( "measure.mesureDate", dateBegin ) );
        if( null != dateEnd )
            criteria.add( Restrictions.le( "measure.mesureDate", dateEnd ) );

        criteria.createCriteria( "localisation", "localisation" );

        criteria.setProjection( Projections.distinct( Projections.projectionList()
                .add( Projections.property( "value.valeurVal" ) )
                .add( Projections.property( "localisation.localisationLat" ) )
                .add( Projections.property( "localisation.localisationLon" ) ) ) );

        criteria.addOrder( Order.asc( "localisation.localisationLon" ) );

//        final List<Object[]> objects = selectAllByCriteria( Object[].class, criteria );
        // TODO : remove maxResults to 10
        final List<Object[]> objects = selectPage( null, 100, criteria );

        final double[] latitudeValues = new double[objects.size()];
        final double[] longitudeValues = new double[objects.size()];
        final double[] parameterValues = new double[objects.size()];

        int i = 0;
        for( final Object[] value : objects )
        {
            parameterValues[i] = (Double) value[0];
            latitudeValues[i] = Nombre.latIntToFloat( (Integer) value[1] );
            longitudeValues[i] = Nombre.latIntToFloat( (Integer) value[2] );
            i++;
        }

        return new Data( parameterValues, latitudeValues, longitudeValues );
    }


    @NotNull
    public <T1, T2, T3> Data<List<T1>, List<T2>, List<T3>> getListsByPlateformByParameterByPeriodFor2D( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( Valeur.class, "value" )
                .add( Restrictions.eq( "value.parametre.id", parameterId ) )
                .createCriteria( "mesure", "measure" )
                .add( Restrictions.eq( "measure.plateforme.id", plateformId ) );

        if( null != dateBegin )
            criteria.add( Restrictions.ge( "measure.mesureDate", dateBegin ) );
        if( null != dateEnd )
            criteria.add( Restrictions.le( "measure.mesureDate", dateEnd ) );

        criteria.createCriteria( "localisation", "localisation" );

        criteria.setProjection( Projections.distinct( Projections.projectionList()
                .add( Projections.property( "value.valeurVal" ) )
                .add( Projections.property( "localisation.localisationLat" ) )
                .add( Projections.property( "localisation.localisationLon" ) ) ) );

        criteria.addOrder( Order.asc( "localisation.localisationLon" ) );

//        final List<Object[]> objects = selectAllByCriteria( Object[].class, criteria );
        // TODO : remove maxResults to 10
        final List<Object[]> objects = selectPage( null, 100, criteria );

        final List<Double> latitudeValues = new ArrayList<Double>( objects.size() );
        final List<Double> longitudeValues = new ArrayList<Double>( objects.size() );
        final List<Double> parameterValues = new ArrayList<Double>( objects.size() );

        int i = 0;
        for( final Object[] value : objects )
        {
            parameterValues.add( (Double) value[0] );
            latitudeValues.add( Double.valueOf( Nombre.latIntToFloat( (Integer) value[1] ) ) );
            longitudeValues.add( Double.valueOf( Nombre.latIntToFloat( (Integer) value[2] ) ) );
            i++;
        }

        return new Data( parameterValues, latitudeValues, longitudeValues );
    }

    @NotNull
    public Integer getNumberValuesByPlateformByParameterByPeriod( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date beginDate, @Nullable final Date endDate )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( Valeur.class, "value" )
                .add( Restrictions.eq( "value.parametre.id", parameterId ) )
                .createCriteria( "mesure", "measure" )
                .add( Restrictions.eq( "measure.plateforme.id", plateformId ) );

        if( null != beginDate )
            criteria.add( Restrictions.ge( "measure.mesureDate", beginDate ) );
        if( null != endDate )
            criteria.add( Restrictions.le( "measure.mesureDate", endDate ) );

        criteria.setProjection( Projections.property( "value.valeurVal" ) );

        return count( criteria );
    }

//    @NotNull
//    public Integer getNumberValuesByPlateformByParameterByPeriod( @NotNull final List<Pair> pairs, @Nullable final Date beginDate, @Nullable final Date endDate )
//            throws PersistenceException
//    {
//        final DetachedCriteria criteria = DetachedCriteria.forClass( Valeur.class, "value" )
//                .add( Restrictions.eq( "value.parametre.id", parameterId ) )
//                .createCriteria( "mesure", "measure" )
//                .add( Restrictions.eq( "measure.plateforme.id", plateformId ) );
//
//        if( null != beginDate )
//            criteria.add( Restrictions.ge( "measure.mesureDate", beginDate ) );
//        if( null != endDate )
//            criteria.add( Restrictions.le( "measure.mesureDate", endDate ) );
//
//        criteria.setProjection( Projections.property( "value.valeurVal" ) );
//
//        return count( criteria );
//    }

}