package com.ether;

import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import com.medias.database.objects.TypePlateforme;
import com.medias.database.objects.Unite;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class BouchonHelper
{
    public static Plateforme createPlateform()
    {
        final Plateforme plateform = new Plateforme();
        plateform.setPlateformeId( 3 );
        plateform.setPlateformeNom( "LHVP" );
        plateform.setTypePlateforme( new TypePlateforme( 1, "FIXE" ) );
        return plateform;
    }

    public static List<Plateforme> createPlateforms()
    {
        final List<Plateforme> plateforms = new ArrayList<Plateforme>();
        final Plateforme pf = createPlateform();
        plateforms.add( pf );

        final Plateforme pf2 = new Plateforme();
        pf2.setPlateformeId( 1 );
        pf2.setPlateformeNom( "SIRTA" );
        pf2.setTypePlateforme( new TypePlateforme( 1, "FIXE" ) );
        plateforms.add( pf2 );

        final Plateforme pf3 = new Plateforme();
        pf3.setPlateformeId( 4 );
        pf3.setPlateformeNom( "ATR-42" );
        pf3.setTypePlateforme( new TypePlateforme( 1, "MOBILE" ) );
        plateforms.add( pf3 );

        final Plateforme pf4 = new Plateforme();
        pf4.setPlateformeId( 6 );
        pf4.setPlateformeNom( "Piper-Aztec" );
        pf4.setTypePlateforme( new TypePlateforme( 1, "MOBILE" ) );
        plateforms.add( pf4 );

        return plateforms;
    }

    public static Plateforme getPlateformById( @NotNull final Integer plateformId )
    {
        if( null != plateformId && plateformId == 1 )
        {
            final Plateforme pf2 = new Plateforme();
            pf2.setPlateformeId( 1 );
            pf2.setPlateformeNom( "SIRTA" );
            pf2.setTypePlateforme( new TypePlateforme( 1, "FIXE" ) );
            return pf2;
        }
        else if( null != plateformId && plateformId == 3 )
        {
            return createPlateform();
        }
        else if( null != plateformId && plateformId == 4 )
        {
            final Plateforme pf3 = new Plateforme();
            pf3.setPlateformeId( 4 );
            pf3.setPlateformeNom( "ATR-42" );
            pf3.setTypePlateforme( new TypePlateforme( 1, "MOBILE" ) );
            return pf3;
        }
        else if( null != plateformId && plateformId == 6 )
        {
            final Plateforme pf4 = new Plateforme();
            pf4.setPlateformeId( 6 );
            pf4.setPlateformeNom( "Piper-Aztec" );
            pf4.setTypePlateforme( new TypePlateforme( 1, "MOBILE" ) );
            return pf4;
        }
        else return null;
    }

    public static Parametre createParameter()
    {
        final Parametre parametre = new Parametre();
        parametre.setParametreId( 3 );
        parametre.setParametreNom( "Ozone" );
        parametre.setParametreCode( "O3" );

        final Unite unit = new Unite();
        unit.setUniteCode( "ppbv" );
        parametre.setUnite( unit );

        return parametre;
    }

    public static List<Parametre> createParameters( @Nullable final Integer plateformId )
    {
        final Unite unitPpbv = new Unite();
        unitPpbv.setUniteCode( "ppbv" );
        final Unite units_1 = new Unite();
        units_1.setUniteCode( "s-1" );
        final Unite unitMbar = new Unite();
        unitMbar.setUniteCode( "mbar" );
        final Unite unitC = new Unite();
        unitC.setUniteCode( "°C" );


        final List<Parametre> parameters = new ArrayList<Parametre>();

        final Parametre pOzone = createParameter();

        final Parametre pJNO2 = new Parametre();
        pJNO2.setParametreId( 1 );
        pJNO2.setParametreNom( "Photolysis Frequency JNO2" );
        pJNO2.setParametreCode( "JNO2" );
        pJNO2.setUnite( units_1 );

        final Parametre pPressure = new Parametre();
        pPressure.setParametreId( 9 );
        pPressure.setParametreNom( "Pressure" );
        pPressure.setParametreCode( "P" );
        pPressure.setUnite( unitMbar );

        final Parametre pTemperature = new Parametre();
        pTemperature.setParametreId( 10 );
        pTemperature.setParametreNom( "Temperature" );
        pTemperature.setParametreCode( "T" );
        pTemperature.setUnite( unitC );

        final Parametre pNO = new Parametre();
        pNO.setParametreId( 4 );
        pNO.setParametreNom( "Nitrogen Monoxide" );
        pNO.setParametreCode( "NO" );
        pNO.setUnite( unitPpbv );

        final Parametre pNOx = new Parametre();
        pNOx.setParametreId( 23 );
        pNOx.setParametreNom( "Nitrogen Oxides" );
        pNOx.setParametreCode( "NOx" );
        pNOx.setUnite( unitPpbv );


        if( null != plateformId && plateformId == 1 )
        {
            // SIRTA
            parameters.add( pOzone );
            parameters.add( pJNO2 );
            parameters.add( pPressure );
            parameters.add( pTemperature );
            parameters.add( pNO );

        }
        else if( null != plateformId && plateformId == 3 )
        {
            // LHVP
            parameters.add( pOzone );
            parameters.add( pNO );
            parameters.add( pNOx );
        }
        else if( null != plateformId && plateformId == 4 )
        {
            // ATR-42
            parameters.add( pPressure );
            parameters.add( pTemperature );
        }
        else if( null != plateformId && plateformId == 6 )
        {
            // Piper-Aztec
        }
        else
        {
            parameters.add( pOzone );
            parameters.add( pJNO2 );
            parameters.add( pPressure );
            parameters.add( pTemperature );
            parameters.add( pNO );
            parameters.add( pNOx );
        }

        return parameters;
    }

    public static Parametre getParameterById( @NotNull final Integer parameterId )
    {
        if( parameterId == 3 )
            return createParameter();

        else if( parameterId == 1 )
        {
            final Parametre pJNO2 = new Parametre();
            pJNO2.setParametreId( 1 );
            pJNO2.setParametreNom( "Photolysis Frequency JNO2" );
            pJNO2.setParametreCode( "JNO2" );
            final Unite units_1 = new Unite();
            units_1.setUniteCode( "s-1" );
            pJNO2.setUnite( units_1 );
            return pJNO2;
        }
        else if( parameterId == 9 )
        {
            final Parametre pPressure = new Parametre();
            pPressure.setParametreId( 9 );
            pPressure.setParametreNom( "Pressure" );
            pPressure.setParametreCode( "P" );
            final Unite unitMbar = new Unite();
            unitMbar.setUniteCode( "mbar" );

            pPressure.setUnite( unitMbar );
            return pPressure;
        }
        else if( parameterId == 10 )
        {
            final Parametre pTemperature = new Parametre();
            pTemperature.setParametreId( 10 );
            pTemperature.setParametreNom( "Temperature" );
            pTemperature.setParametreCode( "T" );
            final Unite unitC = new Unite();
            unitC.setUniteCode( "°C" );
            pTemperature.setUnite( unitC );
            return pTemperature;
        }
        else if( parameterId == 4 )
        {

            final Parametre pNO = new Parametre();
            pNO.setParametreId( 4 );
            pNO.setParametreNom( "Nitrogen Monoxide" );
            pNO.setParametreCode( "NO" );
            final Unite unitPpbv = new Unite();
            unitPpbv.setUniteCode( "ppbv" );
            pNO.setUnite( unitPpbv );
            return pNO;
        }
        else if( parameterId == 23 )
        {

            final Parametre pNOx = new Parametre();
            pNOx.setParametreId( 23 );
            pNOx.setParametreNom( "Nitrogen Oxides" );
            pNOx.setParametreCode( "NOx" );
            final Unite unitPpbv = new Unite();
            unitPpbv.setUniteCode( "ppbv" );
            pNOx.setUnite( unitPpbv );
            return pNOx;
        }
        else return null;
    }

    public static Data createValuesForTimeSerie( final Integer size, final Integer delta )
    {
        if( null == delta )
            return new Data( new double[0], new Date[0] );

        final double[] values = new double[size];
        final Date[] dates = new Date[size];
        final Calendar cal = Calendar.getInstance();

        for( int i = 0; i < size; i++ )
        {
            values[i] = (double) i * Math.random() * delta;
            cal.set( Calendar.MINUTE, (int) ( 2 + i + ( Math.random() * 8 ) ) );
            dates[i] = cal.getTime();
        }
        return new Data( values, dates );
    }

    public static Data createValuesFor2D( final Integer size, final Integer delta )
    {
//                final double[] latitudeArray = {2.1, 2.2, 2.3};
//                final double[] longitudeValues = {112.1, 12.2, 22.3};
        final double[] latitudeValues = new double[size];
        final double[] longitudeValues = new double[size];
        final double[] parameterValues = new double[size];

        for( int i = 0; i < size; i++ )
        {
            parameterValues[i] = Math.random() * i;
            latitudeValues[i] = Math.random() * i;
            longitudeValues[i] = Math.random() * i;
        }

        return new Data( parameterValues, latitudeValues, longitudeValues );
    }

    public static Data createRealValuesFor2D()
    {
//        final double[] latitudeValues = {488993.0, 489001.0, 488984.0, 489010.0, 490005.0, 490015.0};
//        final double[] longitudeValues = {-1829.0, -1829.0, -1828.0, -1828.0, -1827.0, -1826.0};
//        final double[] parameterValues = {7.029, 0.0, 0.0, 0.0, 8.547, 0.0, 0.0, 0.0, 8.845, 0.0, 0.0, 0.0, 9.248, 0.0, 0.0, 0.0, 9.58, 0.0, 0.0, 0.0, 9.98, 0.0, 0.0, 0.0, 7.98, 0.0, 0.0, 0.0, 6.58, 0.0, 0.0, 0.0, 8.08, 0.0, 0.0, 0.0};

        final Double[] latitudeArray = new Double[]{48.898, 48.899, 48.900, 49.1001, 49.1002, 49.1003, 49.1004};
        final Double[] longitudeArray = new Double[]{-0.1832, 2.0368, 2.0369, 2.037, 2.8216, 2.8221, 3.0};
        final Double[] parameterArray = new Double[]{0.1094, 30.8, 19.0, 12.31, 12.69, 12.06, 6.813};

        final List<Double> latitudeValues = Arrays.asList( latitudeArray );
        final List<Double> longitudeValues = Arrays.asList( longitudeArray );
        final List<Double> parameterValues = Arrays.asList( parameterArray );

        return new Data( parameterValues, latitudeValues, longitudeValues );
    }

}
