package com.ether;

import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import gov.noaa.pmel.sgt.Axis;
import gov.noaa.pmel.sgt.CartesianGraph;
import gov.noaa.pmel.sgt.ColorMap;
import gov.noaa.pmel.sgt.ContourLevels;
import gov.noaa.pmel.sgt.Graph;
import gov.noaa.pmel.sgt.GridAttribute;
import gov.noaa.pmel.sgt.IndexedColorMap;
import gov.noaa.pmel.sgt.JPane;
import gov.noaa.pmel.sgt.Layer;
import gov.noaa.pmel.sgt.LineAttribute;
import gov.noaa.pmel.sgt.LineCartesianRenderer;
import gov.noaa.pmel.sgt.LineKey;
import gov.noaa.pmel.sgt.LinearTransform;
import gov.noaa.pmel.sgt.PlainAxis;
import gov.noaa.pmel.sgt.SGLabel;
import gov.noaa.pmel.sgt.StackedLayout;
import gov.noaa.pmel.sgt.TimeAxis;
import gov.noaa.pmel.sgt.demo.TestData;
import gov.noaa.pmel.sgt.dm.SGTData;
import gov.noaa.pmel.sgt.dm.SGTMetaData;
import gov.noaa.pmel.sgt.dm.SimpleGrid;
import gov.noaa.pmel.sgt.dm.SimpleLine;
import gov.noaa.pmel.sgt.swing.JPlotLayout;
import gov.noaa.pmel.util.Dimension2D;
import gov.noaa.pmel.util.GeoDateArray;
import gov.noaa.pmel.util.Point2D;
import gov.noaa.pmel.util.Range2D;
import gov.noaa.pmel.util.SoTPoint;
import gov.noaa.pmel.util.SoTRange;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;

import java.awt.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Random;
import java.util.ResourceBundle;

/**
 * @author vmipsl
 * @date 05 sept. 2011
 */
public class EtherPlotContentServiceImpl
{
    /**
     * First layer contains axes, labels and the first set of data
     * Others layers contains only the other sets of data
     *
     * @param megapoliPlot
     * @param locale
     * @return
     */
    @Nullable
    public JPane createTimeSeriePlot( @NotNull final MegapoliPlot megapoliPlot, @NotNull final Integer plotWidth, @NotNull final Integer plotHeight, final Locale locale )
            throws ServiceException
    {
        final JPane mainPane = new JPane( "Time Serie Pane", new Dimension( plotWidth, plotHeight ) );
        mainPane.setLayout( new StackedLayout() );

        // TODO : revoir les tailles ! mettre en automatique par rapport à plotWidth et plotHeight
        /*
        * xsize, ysize are the width and height in physical units
        * of the Layer graphics region.
        *
        * xstart, xend are the start and end points for the TimeAxis
        * ystart, yend are the start and end points for the Y axis
        */
        final double xsize = 5.0;
        final double ysize = 3.5;
        final double xstart = 0.3;
        final double xend = xsize - xstart;
        final double ystart = 0.5;
        final double yend = ysize - ystart;

        final Layer layer = new Layer( "First Layer", new Dimension2D( xsize, ysize ) );
        mainPane.add( layer );

        final Pair<List, List> listPairOfPIdPIdForTwoAxes = extractListPairForAxes( megapoliPlot.getpIdPIdList() );
        final List<Pair> pIdPIdListForFirstAxis = listPairOfPIdPIdForTwoAxes.getFirstValue();
        final List<Pair> pIdPIdListForSecondAxis = listPairOfPIdPIdForTwoAxes.getSecondValue();

        final List<SimpleLine> allLines = createLines( pIdPIdListForFirstAxis, megapoliPlot.getBeginDate(), megapoliPlot.getEndDate(), 10, locale );
        final List<SimpleLine> allLinesForSecondAxis = createLines( pIdPIdListForSecondAxis, megapoliPlot.getBeginDate(), megapoliPlot.getEndDate(), 50, locale );
        List<SimpleLine> lines = extractRealLines( allLines );
        List<SimpleLine> linesForSecondAxis = extractRealLines( allLinesForSecondAxis );

        final List<SimpleLine> emptyLineToDisplayOnlyInLegend = createEmptyLinesList( allLines, allLinesForSecondAxis, lines );
        if( lines.isEmpty() )
        {
            if( linesForSecondAxis.isEmpty() )
                return null;
            else
            {
                lines = linesForSecondAxis;
                linesForSecondAxis = new ArrayList<SimpleLine>();
            }
        }

        final SimpleLine firstLine = lines.get( 0 );

        /** ********** AXIS RANGES ********* **/
        final SoTRange xnRange = firstLine.getXRange();
        final SoTRange yRange = firstLine.getYRange();
        for( final SimpleLine line : lines )
        {
            yRange.add( line.getYRange() );
            xnRange.add( line.getXRange() );
        }

        for( final SimpleLine line : linesForSecondAxis )
            xnRange.add( line.getXRange() );

        final SoTRange ynRange = Graph.computeRange( yRange, _intervalsNumber );

        // Origin point
        final SoTPoint originPoint = new SoTPoint( ynRange.getStart(), xnRange.getStart() );


        /** ************ AXIS ********** **/
        // Time axis is the same for the two y-axis
        final TimeAxis xTimeAxis = createXTimeAxis( xnRange, originPoint );
        final PlainAxis yLeftAxis = createYAxis( firstLine, ynRange, originPoint, null );

        /** ************ GRAPH ********** **/
        final CartesianGraph graph = new CartesianGraph( "First Graph" );
        layer.setGraph( graph );

        final LinearTransform xt = new LinearTransform( new Range2D( xstart, xend ), xnRange );
        graph.setXTransform( xt );
        final LinearTransform yt = new LinearTransform( new Range2D( ystart, yend ), ynRange );
        graph.setYTransform( yt );

        graph.addXAxis( xTimeAxis );
        graph.addYAxis( yLeftAxis );

        final LineAttribute lineAttribute = createRandomLineAttribute( megapoliPlot.getAxeType() );
        graph.setData( firstLine, lineAttribute );

        /** *********** LEGEND ********* **/
        final LineKey lkey = createLineKey( megapoliPlot, ysize, layer, firstLine, graph );

        /** *********** OTHER LAYERS ********* **/
        // First axis
        addOtherLinesForOneAxis( megapoliPlot.getAxeType(), mainPane, xsize, ysize, lines, xt, yt, lkey, 1 );

        // Second axis
        if( !linesForSecondAxis.isEmpty() )
        {
            final SimpleLine firstLineForSecondAxis = linesForSecondAxis.get( 0 );

            final SoTRange yRangeForSecondAxis = firstLineForSecondAxis.getYRange();
            for( final SimpleLine line : linesForSecondAxis )
                yRangeForSecondAxis.add( line.getYRange() );

            final SoTRange ynRangeForSecondAxis = Graph.computeRange( yRangeForSecondAxis, _intervalsNumber );
            final SoTPoint endPointForSecondAxis = new SoTPoint( ynRangeForSecondAxis.getStart(), xnRange.getEnd() );

            final PlainAxis yRightAxis = createYAxis( firstLineForSecondAxis, ynRangeForSecondAxis, endPointForSecondAxis, Axis.POSITIVE_SIDE );

            final LinearTransform ytForSecondAxis = new LinearTransform( new Range2D( ystart, yend ), ynRangeForSecondAxis );

            // It transforms the graph with the maximum yAxis
            if( ytForSecondAxis.getRangeU().end > yt.getRangeU().end )
                graph.setYTransform( ytForSecondAxis );
            graph.addYAxis( yRightAxis );

            addOtherLinesForOneAxis( megapoliPlot.getAxeType(), mainPane, xsize, ysize, linesForSecondAxis, xt, ytForSecondAxis, lkey, 0 );
        }
        displayLegendForEmptyLines( emptyLineToDisplayOnlyInLegend, lkey, mainPane, xsize, ysize, xt, yt, megapoliPlot.getAxeType() );

        return mainPane;
    }

    @NotNull
    public JPane create2DPlot( @NotNull final MegapoliPlot megapoliPlot, @NotNull final Integer plotWidth, @NotNull final Integer plotHeight, @Nullable final Locale locale )
    {
//        final JPane mainPane = new JPane( "2D Pane", new Dimension( plotWidth, plotHeight ) );

        final ResourceBundle bundle = WebHelper.getBundle( locale );

        final List<Double> parameterValues = (List<Double>) megapoliPlot.getData().getFirstArray();
        final List<Double> latitudeValues = (List<Double>) megapoliPlot.getData().getSecondArray();
        final List<Double> longitudeValues = (List<Double>) megapoliPlot.getData().getThirdArray();

        SGTData newData;
        TestData td;

        /*
       * Create a test grid with sinasoidal-ramp data.
        */
//        Range2D xr = new Range2D( 190.0f, 250.0f, 1.0f );
//        Range2D yr = new Range2D( 0.0f, 45.0f, 1.0f );
        Range2D xr = new Range2D( 190.0f, 200.0f, 1.0f );
        Range2D yr = new Range2D( 0.0f, 4.0f, 1.0f );
//        newData = testDataa( xr, yr, TestData.SINE_RAMP, 12.0f, 30.f, 5.0f, parameterArray, latitudeArray, longitudeValues );


//            public SGTData testDataa( Range2D range1, Range2D range2, int type, float amp, float off, float per, final double[] parameterArray, final double[] latitudeArray, final double[] longitudeValues )
        final SimpleGrid simpleGrid;
        final SGTMetaData xMeta;
        final SGTMetaData yMeta;
        final SGTMetaData zMeta;

        final int num1 = (int) ( ( xr.end - xr.start ) / xr.delta ) + 1;
        final double[] axis1 = new double[num1];
        final int num2 = (int) ( ( yr.end - yr.start ) / yr.delta ) + 1;
        final double[] axis2 = new double[num2];

        for( int count = 0; count < num1; count++ )
            axis1[count] = xr.start + count * xr.delta;

        for( int count = 0; count < num2; count++ )
            axis2[count] = yr.start + count * yr.delta;

//        final double[] valuesOrigPetites = getValues( axis1, num1, axis2, 2, TestData.SINE_RAMP, 12.0f, 30.f, 5.0f );
//        final double[] valuesOrigPetites = getValues( longitudeValues, longitudeValues.length, latitudeValues, latitudeValues.length, TestData.SINE_RAMP, 12.0f, 30.f, 5.0f );

//        final double[] valuesOrig = new double[num1 * num2];
//        for( int count = 0; count < valuesOrigPetites.length; count++ )
//        {
//            valuesOrig[count] = valuesOrigPetites[count];
//        }
//        for( int count = valuesOrigPetites.length; count < num1 * num2; count++ )
//        {
//            valuesOrig[count] = Double.NaN;
//        }

        final double[] parameterValuesArray = EtherHelper.convertListDoubleToDoubleArray( parameterValues );
        final double[] parameterValuesMap = new double[latitudeValues.size() * longitudeValues.size()];
        for( int i = 0; i < parameterValuesArray.length; i++ )
            parameterValuesMap[i] = parameterValuesArray[i];

        for( int i = parameterValuesArray.length; i < latitudeValues.size() * longitudeValues.size(); i++ )
            parameterValuesMap[i] = Double.NaN;

        SGLabel keyLabel = new SGLabel( "Key Label", "", new Point2D.Double( 0.0, 0.0 ) );
        keyLabel.setHeightP( 0.16 );
        //
        // create SimpleGrid
        //
        zMeta = new SGTMetaData( "ts", "m s-1" );
        keyLabel.setText( "XY test grid" );
        xMeta = new SGTMetaData( bundle.getString( "plot.longitude" ), bundle.getString( "plot.degres" ) );
        yMeta = new SGTMetaData( bundle.getString( "plot.latitude" ), bundle.getString( "plot.degres" ) );
//        final SimpleGrid sg = new SimpleGrid( valuesOrig, axis1, axis2, "Test Series" );
//        simpleGrid = new SimpleGrid( parameterValues, longitudeValues, latitudeValues, "Test Series" );
//        simpleGrid = new SimpleGrid( valuesOrig, longitudeValues, latitudeValues, "Test Series" );
//        simpleGrid = new SimpleGrid( valuesOrig, axis1, axis2, "Test Series" );

        final double[] longitudeValuesArray = EtherHelper.convertListDoubleToDoubleArray( longitudeValues );
        final double[] latitudeValuesArray = EtherHelper.convertListDoubleToDoubleArray( latitudeValues );
        simpleGrid = new SimpleGrid( parameterValuesMap, longitudeValuesArray, latitudeValuesArray, "Test Series" );

        simpleGrid.setXMetaData( xMeta );
        simpleGrid.setYMetaData( yMeta );
        simpleGrid.setZMetaData( zMeta );
        simpleGrid.setKeyTitle( keyLabel );
        simpleGrid.setRealFullGrid( false );


        // Create the contour levels and color map
        final Double min = Collections.min( parameterValues );
        final Double max = Collections.max( parameterValues );
        final Double delta = ( max - min ) / 64;
        final Range2D datar = new Range2D( min, max, delta );
//        final Range2D datar = new Range2D( Collections.min( Arrays.asList( parameterValuesMap ) ), 35f, 0.5f );
        final ContourLevels clevels = ContourLevels.getDefault( datar );
        final ColorMap cmap = createColorMap( datar );

//        final Range2D datarOrig = new Range2D( 5f, 10.0f, 1f );
//        final ContourLevels clevelsOrig = ContourLevels.getDefault( datarOrig );
//        final ColorMap cmapOrig = createColorMap( datarOrig );
//        final GridAttribute gridAttributeOrig = new GridAttribute( clevelsOrig );
//        gridAttributeOrig.setColorMap( cmapOrig );
//        gridAttributeOrig.setStyle( GridAttribute.RASTER_CONTOUR );


        // Create a grid for contour data
        final GridAttribute gridAttribute = new GridAttribute( clevels );
        gridAttribute.setColorMap( cmap );
        gridAttribute.setStyle( GridAttribute.RASTER );

        // Create the plot
        final JPlotLayout jPlotLayout = new JPlotLayout( true, false, false, "JPlotLayout Pane", null, true );
        jPlotLayout.setEditClasses( false );
        jPlotLayout.addData( simpleGrid, gridAttribute, "First Data" );
        jPlotLayout.setTitles( "", "", "" );
        jPlotLayout.setSize( new Dimension( plotWidth, plotHeight ) );

        /*
       * Resize the key Pane, both the device size and the physical
       * size. Set the size of the key in physical units and place
       * the key pane at the "South" of the frame.
        */
//    rpl.setKeyLayerSizeP(new Dimension2D(6.0, 1.02));
//    rpl.setKeyBoundsP(new Rectangle2D.Double(0.01, 1.01, 5.98, 1.0));

        return jPlotLayout;
//        return mainPane;
    }

    @NotNull
    protected String formatTitle( @Nullable final String title )
    {
        if( null == title )
            return "";
        else
        {
            final String formatTitle = title.replaceAll( "\\n", "<br>" );
            return "<html>" + formatTitle + "</html>";
        }
    }

    private List<SimpleLine> createEmptyLinesList( final List<SimpleLine> allLines, final List<SimpleLine> allLinesForSecondAxis, final List<SimpleLine> lines )
    {
        final List<SimpleLine> emptyLinesList = allLines;
        emptyLinesList.removeAll( lines );
        emptyLinesList.addAll( allLinesForSecondAxis );
        emptyLinesList.removeAll( allLinesForSecondAxis );

        return emptyLinesList;
    }

    private void displayLegendForEmptyLines( @NotNull final List<SimpleLine> lines, @NotNull final LineKey lkey, @NotNull final JPane mainPane, final double xsize, final double ysize, final LinearTransform xt, final LinearTransform yt, final String axeType )
    {
        for( final SimpleLine line : lines )
        {
            final Layer otherLayer = new Layer( "Other Layer", new Dimension2D( xsize, ysize ) );
            mainPane.add( otherLayer );
            final CartesianGraph otherGraph = new CartesianGraph( "Other Graph", xt, yt );
            otherLayer.setGraph( otherGraph );

            final LineAttribute otherLineAttribute = createRandomLineAttribute( axeType );
            otherGraph.setData( line, otherLineAttribute );
            lkey.addLineGraph( (LineCartesianRenderer) otherGraph.getRenderer(), line.getKeyTitle() );
        }
    }

    /**
     * This method removes from a list the lines with no data
     *
     * @param allLines
     * @return
     */
    @NotNull
    private List<SimpleLine> extractRealLines( @NotNull final List<SimpleLine> allLines )
    {
        final List<SimpleLine> lines = new ArrayList<SimpleLine>();
        for( final SimpleLine line : allLines )
            if( 0 < line.getYArray().length )
                lines.add( line );

        return lines;
    }

    /**
     * This method creates a legend
     *
     * @param megapoliPlot
     * @param ysize
     * @param layer
     * @param firstLine
     * @param graph
     * @return
     */
    @NotNull
    private LineKey createLineKey( final MegapoliPlot megapoliPlot, final double ysize, final Layer layer, final SimpleLine firstLine, final CartesianGraph graph )
    {
        final LineKey lkey = new LineKey();
        if( !megapoliPlot.isLegendToHide() )
        {
            lkey.setId( "Legend" );
            lkey.setLocationP( new Point2D.Double( 0.1, ysize - 0.1 ) );
            lkey.setBorderStyle( LineKey.NO_BORDER );
            lkey.setVAlign( LineKey.TOP );
            lkey.setHAlign( LineKey.LEFT );
            layer.addChild( lkey );
            lkey.addLineGraph( (LineCartesianRenderer) graph.getRenderer(), firstLine.getKeyTitle() );
        }
        return lkey;
    }

    /**
     * This method creates a x-axis for time serie
     *
     * @param xnRange
     * @param originPoint
     * @return
     */
    @NotNull
    private TimeAxis createXTimeAxis( final SoTRange xnRange, final SoTPoint originPoint )
    {
        final TimeAxis xbot = new TimeAxis( "X-Axis", TimeAxis.AUTO );
        xbot.setRangeU( xnRange );
        xbot.setLocationU( originPoint );
        xbot.setLabelFont( _axisFont );
        xbot.setLabelHeightP( _heightAxisFont );
        xbot.setMajorLabelFormat( _majorLabelFormat );
        return xbot;
    }

    /**
     * This method adds layers corresponding to the other lines to display
     *
     * @param axeType
     * @param mainPane
     * @param xsize
     * @param ysize
     * @param lines
     * @param xt
     * @param yt
     * @param lkey
     * @param indexLineToBegin
     */
    private void addOtherLinesForOneAxis( @NotNull final String axeType, @NotNull final JPane mainPane, final double xsize, final double ysize, @NotNull final List<SimpleLine> lines, @NotNull final LinearTransform xt, @NotNull final LinearTransform yt, @NotNull final LineKey lkey, @NotNull final Integer indexLineToBegin )
    {
        for( final SimpleLine line : lines.subList( indexLineToBegin, lines.size() ) )
        {
            final Layer otherLayer = new Layer( "Other Layer", new Dimension2D( xsize, ysize ) );
            mainPane.add( otherLayer );
            final CartesianGraph otherGraph = new CartesianGraph( "Other Graph", xt, yt );
            otherLayer.setGraph( otherGraph );

            final LineAttribute otherLineAttribute = createRandomLineAttribute( axeType );
            otherGraph.setData( line, otherLineAttribute );
            lkey.addLineGraph( (LineCartesianRenderer) otherGraph.getRenderer(), line.getKeyTitle() );
        }
    }

    /**
     * This method creates a vertical axis, set its range in user units and its origin and creates the axis title
     *
     * @param line
     * @param ynRange
     * @param originPoint
     * @param labelPosition
     * @return
     */
    @NotNull
    private PlainAxis createYAxis( final SimpleLine line, final SoTRange ynRange, final SoTPoint originPoint, @Nullable final Integer labelPosition )
    {
        final PlainAxis yAxis = new PlainAxis( "Y-Axis" );
        yAxis.setRangeU( ynRange );
        yAxis.setLocationU( originPoint );
        yAxis.setLabelFont( _axisFont );
        if( null != labelPosition )
            yAxis.setLabelPosition( labelPosition );

        final String yLabel = line.getYMetaData().getName() + " (" + line.getYMetaData().getUnits() + ")";
        final SGLabel ytitle = new SGLabel( "Y-Axis Title", yLabel, new Point2D.Double( 0.0, 0.0 ) );
        ytitle.setFont( _axisFont );
        ytitle.setHeightP( _heightAxisFont );
        yAxis.setTitle( ytitle );
        return yAxis;
    }

    /**
     * This method returns a new LineAttribute with a random color
     *
     * @param axeType
     * @return
     */
    @NotNull
    private LineAttribute createRandomLineAttribute( @NotNull final String axeType )
    {
        final LineAttribute attr;
        final Color color = new Color( new Random().nextInt( 256 ), new Random().nextInt( 256 ), new Random().nextInt( 256 ) );

        if( axeType.equals( AxeTypeForFixedPlateform.TIME_POINTS.toString() ) )
            attr = new LineAttribute( LineAttribute.MARK, 1, color );
        else
            attr = new LineAttribute( LineAttribute.SOLID, 1, color );

        attr.setMarkHeightP( _markHeight );

        return attr;
    }

    /**
     * This method extracts from a list of <plateformId, parameterId> two lists of <plateformId, parameterId> corresponding for the 2 axes needed for the graph
     * The second list can be empty, in this case, only one axis is needed
     *
     * @param pIdPIdList
     * @return
     */
    @NotNull
    private Pair<List, List> extractListPairForAxes( @NotNull final List<Pair> pIdPIdList )
    {
        final List<Pair<Integer, Integer>> pIdPIdListForFirstAxis = new ArrayList<Pair<Integer, Integer>>();
        final List<Pair<Integer, Integer>> pIdPIdListForSecondAxis = new ArrayList<Pair<Integer, Integer>>();

        for( final Pair pIdPId : pIdPIdList )
        {
            final List<Integer> secondValuesForFirstAxis = EtherHelper.getSecondValues( pIdPIdListForFirstAxis );
            final Integer parameterId = (Integer) pIdPId.getSecondValue();

            if( pIdPIdListForFirstAxis.isEmpty() || secondValuesForFirstAxis.contains( parameterId ) )
                pIdPIdListForFirstAxis.add( pIdPId );
            else
                pIdPIdListForSecondAxis.add( pIdPId );
        }

        return new Pair( pIdPIdListForFirstAxis, pIdPIdListForSecondAxis );
    }

    /**
     * This method extracts data from BD for the plateform and parameter given as parameters
     * and creates the corresponded simpleLine
     * Some lines can be empty if there's no data in this period by example
     *
     * @param pIdPIdList
     * @param beginDate
     * @param endDate
     * @param locale
     * @return
     * @throws ServiceException
     */
    @NotNull
    private List<SimpleLine> createLines( @NotNull final List<Pair> pIdPIdList, @Nullable final Date beginDate, @Nullable final Date endDate, final Integer delta, final Locale locale )
            throws ServiceException
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String messageData = bundle.getString( "plot.noData" );

        final List<SimpleLine> lines = new ArrayList<SimpleLine>( pIdPIdList.size() );
        boolean isFirstLine = true;

        for( final Pair<Integer, Integer> pIdPId : pIdPIdList )
        {
            final Integer plateformId = pIdPId.getFirstValue();
            final Integer parameterId = pIdPId.getSecondValue();

//            final Plateforme plateform = _etherService.getPlateformById( plateformId );
            final Plateforme plateform = BouchonHelper.getPlateformById( plateformId );
            if( null == plateform )
                throw new ServiceException( ServiceException.ServiceCode.PLATEFORM_IS_NULL, new Throwable( ServiceException.ServiceCode.PLATEFORM_IS_NULL.toString() ) );

//            final Parametre parametre = _etherService.getParameterById( parameterId );
            final Parametre parametre = BouchonHelper.getParameterById( parameterId );
            if( null == parametre )
                throw new ServiceException( ServiceException.ServiceCode.PARAMETER_IS_NULL, new Throwable( ServiceException.ServiceCode.PARAMETER_IS_NULL.toString() ) );

//            final Data valuesLists = _etherService.getListsByPlateformByParameterByPeriodForTimeSerie( plateformId, parameterId, beginDate, endDate );
            final Data valuesLists = BouchonHelper.createValuesForTimeSerie( 50, delta );

            final double[] dataArray = (double[]) valuesLists.getFirstArray();
            final Date[] dateValues = (Date[]) valuesLists.getSecondArray();

            final SimpleLine line = new SimpleLine( new GeoDateArray( dateValues ), dataArray, "legend" );
            SGTMetaData meta = new SGTMetaData( "", "", false, false );
            line.setXMetaData( meta );
            final SGLabel sgLabel;
            if( 0 < dataArray.length && 0 < dateValues.length )
                sgLabel = new SGLabel( "Line", plateform.getPlateformeNom() + "-" + parametre.getParametreNom(), new Point2D.Double( 0.0, 0.0 ) );
            else
                sgLabel = new SGLabel( "Line", plateform.getPlateformeNom() + "-" + parametre.getParametreNom() + " " + messageData, new Point2D.Double( 0.0, 0.0 ) );
            line.setKeyTitle( sgLabel );

            if( isFirstLine )
                meta = new SGTMetaData( parametre.getParametreNom(), parametre.getUnite().getUniteCode(), false, false );
            else
                meta = new SGTMetaData( "", "", false, false );
            line.setYMetaData( meta );

            lines.add( line );
            isFirstLine = false;
        }
        return lines;
    }

    private double[] getValues( double[] axis1, int num1, double[] axis2, int num2,
                                int type, float amp, float off, float per )
    {
        double[] values;
        int count1, count2, count;
        values = new double[num1 * num2];
        switch( type )
        {
            default:
            case TestData.RANDOM:
                for( count = 0; count < num1 * num2; count++ )
                {
                    values[count] = amp * Math.random() + off;
                }
                break;
            case TestData.SINE:
                count = 0;
                for( count1 = 0; count1 < num1; count1++ )
                {
                    for( count2 = 0; count2 < num2; count2++ )
                    {
                        values[count] = amp * Math.sin( axis1[count1] / per ) *
                                Math.sin( axis2[count2] / per ) + off;
                        count++;
                    }
                }
                break;
            case TestData.SINE_RAMP:
                count = 0;
                double ax1factr = 0.08 * Math.abs( axis1[0] - axis1[num1 - 1] ) /
                        Math.max( Math.abs( axis1[0] ), Math.abs( axis1[num1 - 1] ) );
                double ax2factr = 0.08 * Math.abs( axis2[0] - axis2[num2 - 1] ) /
                        Math.max( Math.abs( axis2[0] ), Math.abs( axis2[num2 - 1] ) );
                for( count1 = 0; count1 < num1; count1++ )
                {
                    for( count2 = 0; count2 < num2; count2++ )
                    {
                        values[count] = amp * Math.sin( axis1[count1] / per ) *
                                Math.sin( axis2[count2] / per ) + off +
                                amp * ( ax1factr * count1 - ax2factr * count2 );
                        count++;
                    }
                }
        }
        return values;
    }


    @NotNull
    private ColorMap createColorMap( final Range2D datar )
    {
        final int[] red =
                {0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 7, 23, 39, 55, 71, 87, 103,
                        119, 135, 151, 167, 183, 199, 215, 231,
                        247, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 246, 228, 211, 193, 175, 158, 140};
        final int[] green =
                {0, 0, 0, 0, 0, 0, 0, 0,
                        0, 11, 27, 43, 59, 75, 91, 107,
                        123, 139, 155, 171, 187, 203, 219, 235,
                        251, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 247, 231, 215, 199, 183, 167, 151,
                        135, 119, 103, 87, 71, 55, 39, 23,
                        7, 0, 0, 0, 0, 0, 0, 0};
        final int[] blue =
                {0, 143, 159, 175, 191, 207, 223, 239,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 247, 231, 215, 199, 183, 167, 151,
                        135, 119, 103, 87, 71, 55, 39, 23,
                        7, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0};

        final IndexedColorMap cmap = new IndexedColorMap( red, green, blue );
        cmap.setTransform( new LinearTransform( 0.0, (double) red.length, datar.start, datar.end ) );
        return cmap;
    }

    public EtherService getEtherService()
    {
        return _etherService;
    }

    @Required
    public void setEtherService( final EtherService etherService )
    {
        _etherService = etherService;
    }

    // Dimensions of the jPanes
    protected static final int MAX_WIDTH = 800;
    protected static final int MAX_HEIGHT = 700;
    protected static final int MARGIN_LEFT_RIGHT = 50;

    protected static final int TITLE_FONT_SIZE = 20;
    protected static final int FONT_SIZE = 12;
    protected static final int ERROR_FONT_SIZE = 10;

    protected static final Font _axisFont = new Font( "Helvetica", Font.PLAIN, 15 );
    protected static final double _heightAxisFont = 0.14;
    protected static final double _markHeight = 0.1;
    protected static final String _majorLabelFormat = "yyyy-MM-dd HH:mm";
    // Number of intervals on a Y-Axis
    protected Integer _intervalsNumber = 10;

    private EtherService _etherService;
}
