package com.ether;

import com.medias.megapoli.utils.MegapoliInitialisation;
import gov.noaa.pmel.sgt.JPane;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Calendar;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * @author vmipsl
 * @date 05 july 2011
 */
public class EtherPlotServiceImpl
        extends EtherPlotContentServiceImpl
        implements EtherPlotService
{
    /**
     * This method returns the maximum width available to display the quicklook in terms of screenSize and default maximum width
     *
     * @return
     */
    public static Integer getMaxWidth()
    {
        final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        return Math.min( Double.valueOf( screenSize.getWidth() ).intValue(), MAX_WIDTH );
    }

    /**
     * This method returns the maximum height available to display the quicklook in terms of screenSize and default maximum height
     *
     * @return
     */
    public static Integer getMaxHeight()
    {
        final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        return Math.min( Double.valueOf( screenSize.getHeight() ).intValue(), MAX_HEIGHT );
    }

    /**
     * Create the main JPane with the 3 jPanes : top, center (graph) et bottom
     *
     * @param megapoliPlot
     * @param locale
     * @return
     */
    @NotNull
    public BufferedImage createMainPane( @NotNull final MegapoliPlot megapoliPlot, @Nullable final Locale locale )
            throws ServiceException
    {
        final Integer maxWidth = getMaxWidth();
        final Integer maxHeight = getMaxHeight();

        final JPane jPane = new JPane( "Main jPane", new Dimension( maxWidth, maxHeight ) );
        jPane.setLayout( new BorderLayout() );
        jPane.setBackground( Color.white );
        jPane.setOpaque( true );

        // Top Pane
        final JPane jPaneTop = createTopPane( megapoliPlot );
        jPane.add( jPaneTop, BorderLayout.NORTH );

        final Integer valuesNumber = ( null != megapoliPlot.getValuesNumber() ) ? megapoliPlot.getValuesNumber() : getEtherService().getNumberValuesByPlateformByParameterByPeriod( megapoliPlot.getpIdPIdList(), megapoliPlot.getBeginDate(), megapoliPlot.getEndDate() );

        // Bottom Pane
        final JPane jPaneBottom = createBottomPane( valuesNumber, locale );
        jPane.add( jPaneBottom, BorderLayout.SOUTH );

        // Center Pane
        final Integer centerHeight = maxHeight - jPaneTop.getHeight() - jPaneBottom.getHeight();
        if( valuesNumber > 0 )
        {
            final JPane jPaneCenter = createCenterPane( megapoliPlot, maxWidth, centerHeight, locale );

            if( null == jPaneCenter )
            {
                final JPane emptyJPaneCenter = createEmptyCenterPane( locale );
                jPane.add( emptyJPaneCenter, BorderLayout.CENTER );
            }
            else
            {
                final BufferedImage jPaneCenterBufferedImage = new BufferedImage( jPaneCenter.getWidth(), jPaneCenter.getHeight(), BufferedImage.TYPE_INT_ARGB );
                final Graphics2D graphics2D = jPaneCenterBufferedImage.createGraphics();
                jPaneCenter.addNotify();
                jPaneCenter.validate();
                jPaneCenter.draw( graphics2D );

                final ImageIcon imageIcon = new ImageIcon( jPaneCenterBufferedImage );
                final JLabel jLabelPlot = new JLabel( imageIcon, JLabel.CENTER );
                jLabelPlot.setBorder( BorderFactory.createLineBorder( Color.black, 1 ) );

                final JPane jPaneCenterToImage = new JPane( "Center jPane", new Dimension( jPaneCenterBufferedImage.getWidth(), jPaneCenterBufferedImage.getHeight() ) );
                jPaneCenterToImage.setLayout( new GridBagLayout() );
                jPaneCenterToImage.add( jLabelPlot,
                        new GridBagConstraints( 0, 0, 1, 1, 0, 0,
                                GridBagConstraints.CENTER,
                                GridBagConstraints.CENTER,
                                new Insets( 0, 0, 0, 0 ), 0, 0 ) );
                jPane.add( jPaneCenterToImage, BorderLayout.CENTER );
            }
        }
        else
        {
            final JPane jPaneCenter = createEmptyCenterPane( locale );
            jPane.add( jPaneCenter, BorderLayout.CENTER );
        }

        final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPane );
        return jPanelToImageManager.saveAsImage();
    }

    /**
     * Create the top JPane with the logos and title
     *
     * @param megapoliPlot
     * @return
     */
    @NotNull
    public JPane createTopPane( @Nullable final MegapoliPlot megapoliPlot )
    {
        // Logos
        final ImageIcon logoMegapoli = new ImageIcon( megapoliPlot.getLogoMegapoli() );
        final JLabel jLabelLogoMegapoli = new JLabel( logoMegapoli );

        final ImageIcon logoEther = new ImageIcon( megapoliPlot.getLogoEther() );
        final JLabel jLabelLogoEther = new JLabel( logoEther );

        // Max height
        final Integer maxHeights = Math.max( logoMegapoli.getIconHeight(), logoEther.getIconHeight() );

        // jPane
        final JPane jPaneTop = new JPane( "Top jPane", new Dimension( 0, maxHeights ) );
        jPaneTop.setLayout( new BorderLayout() );

        jPaneTop.add( jLabelLogoMegapoli, BorderLayout.LINE_START );
        jPaneTop.add( jLabelLogoEther, BorderLayout.LINE_END );


        // Title
        final String formatedTitle = formatTitle( megapoliPlot.getTitle() );
        final JLabel jLabelTitle = new JLabel( formatedTitle, JLabel.CENTER );
        final Font police = new Font( "Arial", Font.BOLD, TITLE_FONT_SIZE );
        jLabelTitle.setFont( police );
        jPaneTop.add( jLabelTitle, BorderLayout.CENTER );

        return jPaneTop;
    }

    /**
     * Create the bottom JPane with the text "published .. (date)"
     *
     * @param dataNumber : the number of datas extracted from the base
     * @param locale
     * @return
     */
    @NotNull
    public JPane createBottomPane( @Nullable final Integer dataNumber, @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );

        final JPane jPaneBottom = new JPane();
        jPaneBottom.setLayout( new BorderLayout() );

        // Published
        final Calendar calendar = Calendar.getInstance();
        final String messagePublished = bundle.getString( "plot.published" );
        final JLabel jLabel = new JLabel( messagePublished + ' ' + DateHelper.formatDate( calendar.getTime(), DateHelper.ENGLISH_DATE_PATTERN ) + " " );
        final Font police = new Font( "Arial", Font.BOLD, FONT_SIZE );
        jLabel.setFont( police );
        jPaneBottom.add( jLabel, BorderLayout.EAST );

        if( null != dataNumber )
        {
            final String messageDataNumber = bundle.getString( "plot.dataNumber" );
            final JLabel jLabelDataNumber = new JLabel( messageDataNumber + ' ' + dataNumber );
            jLabelDataNumber.setFont( police );
            jPaneBottom.add( jLabelDataNumber, BorderLayout.WEST );
        }

        return jPaneBottom;
    }

    /**
     * Create an error JPane when plot can be created
     *
     * @param mainError
     * @param errorText
     * @param locale
     * @return
     */
    @NotNull
    public BufferedImage createErrorPane( @NotNull final String mainError, @Nullable final String errorText, @Nullable final Locale locale )
    {
        // Image
        final ImageIcon errorImage = new ImageIcon( MegapoliInitialisation.pathImages + "/erreur.gif" );
        final JLabel jLabelErrorImage = new JLabel( errorImage );

        // Text
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String mainMessage = bundle.getString( "plot.errorMessage" );

        final JLabel jLabelError;
        if( null != errorText )
            jLabelError = new JLabel( "<html>" + mainMessage + "<br><br>" + mainError + "<br>" + errorText + "</html>", JLabel.CENTER );
        else
            jLabelError = new JLabel( "<html>" + mainMessage + "<br><br>" + mainError + "</html>", JLabel.CENTER );

        final Font police = new Font( "Arial", Font.BOLD, ERROR_FONT_SIZE );
        jLabelError.setFont( police );

        // jPane
        final JPane jPane = new JPane( "jPane", new Dimension( MAX_WIDTH, errorImage.getIconHeight() ) );
        jPane.setLayout( new BorderLayout() );
        jPane.setBackground( Color.white );
        jPane.setOpaque( true );
        jPane.add( jLabelErrorImage, BorderLayout.WEST );
        jPane.add( jLabelError, BorderLayout.CENTER );

        final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPane );
        return jPanelToImageManager.saveAsImage();
    }

    /**
     * Create the center JPane with no graph (if no value)
     *
     * @param locale
     * @return
     */
    @NotNull
    public JPane createEmptyCenterPane( @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String messageData = bundle.getString( "plot.noData" );

        final JPane jPaneCenter = new JPane();
        jPaneCenter.setLayout( new BorderLayout() );

        final JLabel jLabelData = new JLabel( messageData, JLabel.CENTER );
        final Font police = new Font( "Arial", Font.BOLD, FONT_SIZE );
        jLabelData.setFont( police );

        jPaneCenter.add( jLabelData, BorderLayout.CENTER );
        return jPaneCenter;
    }

    /**
     * Create the center JPane with the graph
     *
     * @param megapoliPlot
     * @param maxWidth
     * @param maxHeight
     * @return
     */
    @Nullable
    public JPane createCenterPane( @NotNull final MegapoliPlot megapoliPlot, @NotNull final Integer maxWidth, @NotNull final Integer maxHeight, @Nullable final Locale locale )
            throws ServiceException
    {
        final Integer plotWidth = Math.min( maxWidth - MARGIN_LEFT_RIGHT, maxWidth );
        final Integer plotHeight = Math.min( maxHeight - MARGIN_LEFT_RIGHT, maxHeight );

        final JPane jPaneGraph;

        if( megapoliPlot.isTimeSerie() )
            jPaneGraph = createTimeSeriePlot( megapoliPlot, plotWidth, plotHeight, locale );
        else
            jPaneGraph = create2DPlot( megapoliPlot, plotWidth, plotHeight, locale );

        return jPaneGraph;
    }

    private static final Log LOGGER = LogFactory.getLog( EtherPlotServiceImpl.class );
}
