package com.ether;

import com.ether.dao.ParameterDAO;
import com.ether.dao.PlateformDAO;
import com.ether.dao.ValueDAO;
import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @author vmipsl
 * @date 07 mar 2011
 */
public class EtherServiceImpl
        implements EtherService
{
    @Nullable
    @Transactional(readOnly = true)
    public List<Plateforme> getAllPlateforms()
            throws ServiceException
    {
        try
        {
            return _plateformDAO.getAllPlateforms();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PLATEFORM_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Plateforme getPlateformById( @Nullable final Integer plateformId )
            throws ServiceException
    {
        if( null == plateformId )
            return null;
        try
        {
            return _plateformDAO.getPlateformById( plateformId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PLATEFORM_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Parametre getParameterById( @Nullable final Integer parameterId )
            throws ServiceException
    {
        if( null == parameterId )
            return null;
        try
        {
            return _parameterDAO.getParameterById( parameterId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PARAMETER_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Parametre> getParametersByPlateformId( @NotNull final Integer plateformId )
            throws ServiceException
    {
        try
        {
            return _parameterDAO.getParametersByPlateformId( plateformId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PARAMETER_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public <T1, T2, T3> Data<T1[], T2[], T3[]> getListsByPlateformByParameterByPeriodForTimeSerie( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException
    {
        try
        {
            return _valueDAO.getListsByPlateformByParameterByPeriodForTimeSerie( plateformId, parameterId, dateBegin, dateEnd );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VALUE_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public <T1, T2, T3> Data<List<T1>, List<T2>, List<T3>> getListsByPlateformByParameterByPeriodFor2D( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException
    {
        try
        {
            return _valueDAO.getListsByPlateformByParameterByPeriodFor2D( plateformId, parameterId, dateBegin, dateEnd );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VALUE_NOT_FOUND, e );
        }
    }


    @NotNull
    @Transactional(readOnly = true)
    public <T1, T2, T3> Data<T1[], T2[], T3[]> getArraysByPlateformByParameterByPeriodFor2D( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException
    {
        try
        {
            return _valueDAO.getArraysByPlateformByParameterByPeriodFor2D( plateformId, parameterId, dateBegin, dateEnd );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VALUE_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public Integer getNumberValuesByPlateformByParameterByPeriod( @NotNull final List<Pair> pairs, @Nullable final Date beginDate, @Nullable final Date endDate )
            throws ServiceException
    {
        try
        {
            Integer number = 0;
            for( final Pair pair : pairs )
                number += _valueDAO.getNumberValuesByPlateformByParameterByPeriod( (Integer) pair.getFirstValue(), (Integer) pair.getSecondValue(), beginDate, endDate );

            return number;
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VALUE_NOT_FOUND, e );
        }
    }

    @Required
    public void setPlateformDAO( final PlateformDAO plateformDAO )
    {
        _plateformDAO = plateformDAO;
    }

    @Required
    public void setParameterDAO( final ParameterDAO parameterDAO )
    {
        _parameterDAO = parameterDAO;
    }

    @Required
    public void setValueDAO( final ValueDAO valueDAO )
    {
        _valueDAO = valueDAO;
    }

    private static final Log LOGGER = LogFactory.getLog( EtherServiceImpl.class );

    private PlateformDAO _plateformDAO;
    private ParameterDAO _parameterDAO;
    private ValueDAO _valueDAO;
}
