package com.ether;

import org.apache.commons.lang.ArrayUtils;
import org.junit.Assert;
import org.junit.Test;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author vmipsl
 * @date 2011
 */
public class OtherTest
{
    @Test
    public void testDatesMoisPlusUn()
            throws Exception
    {
        final DateFormat shortDateFormat = new SimpleDateFormat( "MMyyyy" );
        final DateFormat completDateFormat = new SimpleDateFormat( "dd-MM-yyyy" );
        final Date firstMois = shortDateFormat.parse( "102008" );
        final String firstMoisString = completDateFormat.format( firstMois );
        Assert.assertEquals( firstMoisString, "01-10-2008" );

        final Calendar calendar = Calendar.getInstance();
        calendar.setTime( firstMois );
        calendar.add( Calendar.MONTH, 1 );
        final Date secondMois = calendar.getTime();
        final String secondMoisString = completDateFormat.format( secondMois );
        Assert.assertEquals( secondMoisString, "01-11-2008" );
    }

    @Test
    public void testDates()
            throws Exception
    {
        List<Date> dates = new ArrayList<Date>();
        final DateFormat dateFormat = new SimpleDateFormat( "yyyy-MM-dd" );

        dates.add( dateFormat.parse( "2007-01-01" ) );
        dates.add( dateFormat.parse( "2007-01-02" ) );

        final DateFormat shortFormat = new SimpleDateFormat( "yyyy" );
        Set<String> shortDates = new HashSet<String>();
        for( final Date aDate : dates )
        {
            shortDates.add( shortFormat.format( aDate ) );
        }
    }

    /**
     * Test transform List<Pair> en Pair<List, List>
     * Method used in getListsByPlateformByParameterByPeriodForTimeSerie
     */
    @Test
    public void testListPairToPairLists()
    {
        final List<Object[]> objects = new ArrayList<Object[]>( 3 );
        final Object[] obj1 = new Object[2];
        obj1[0] = 2.2;
        obj1[1] = new Date();
        objects.add( obj1 );

        final Object[] obj2 = new Object[2];
        obj2[0] = 2.5;
        obj2[1] = new Date();
        objects.add( obj2 );

        final Object[] obj3 = new Object[2];
        obj3[0] = 2.7;
        obj3[1] = new Date();
        objects.add( obj3 );

        if( objects.get( 0 )[0] instanceof Double && objects.get( 0 )[1] instanceof Date )
        {
            final double[] firstValues = new double[objects.size()];
            final Date[] secondValues = new Date[objects.size()];
            int i = 0;
            for( final Object[] value : objects )
            {
                secondValues[i] = (Date) value[1];
                firstValues[i] = (Double) value[0];
                i++;
            }
            final Pair<double[], Date[]> values = new Pair( firstValues, secondValues );

            Assert.assertEquals( values.getFirstValue().length, objects.size() );
            Assert.assertEquals( values.getSecondValue().length, objects.size() );
            Assert.assertEquals( values.getFirstValue()[0], obj1[0] );
            Assert.assertEquals( values.getSecondValue()[0], obj1[1] );
            Assert.assertEquals( values.getFirstValue()[1], obj2[0] );
            Assert.assertEquals( values.getSecondValue()[1], obj2[1] );
            Assert.assertEquals( values.getFirstValue()[2], obj3[0] );
            Assert.assertEquals( values.getSecondValue()[2], obj3[1] );
        }
    }

    @Test
    public void testConversionListToArray()
    {
        final List<Double> doubleList = new ArrayList<Double>( 2 );
        doubleList.add( 1.2 );
        doubleList.add( 5.6 );

        Double[] doubleArray = new Double[doubleList.size()];
        doubleArray = doubleList.toArray( doubleArray );

        final double[] doubles = ArrayUtils.toPrimitive( doubleArray );
        Assert.assertEquals( doubles[0], (double) doubleList.get( 0 ), 0 );
        Assert.assertEquals( doubles[1], (double) doubleList.get( 1 ), 0 );

        double[] doubles1 = EtherHelper.convertListDoubleToDoubleArray( doubleList );
        Assert.assertEquals( doubles1[0], (double) doubleList.get( 0 ), 0 );
        Assert.assertEquals( doubles1[1], (double) doubleList.get( 1 ), 0 );
    }
}