package com.ether;

import com.medias.Context;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import javax.imageio.ImageIO;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @author vmipsl
 * @date 17 june 2011
 */
public class ControllerPlot
        extends HttpServlet
{
    public void init( final ServletConfig servletConfig )
            throws ServletException
    {
        try
        {
            final ApplicationContext appContext = WebApplicationContextUtils.getRequiredWebApplicationContext( servletConfig.getServletContext() );
            _etherPlotService = (EtherPlotService) appContext.getBean( "etherPlotService", EtherPlotService.class );
            _etherService = (EtherService) appContext.getBean( "etherService", EtherService.class );
        }
        catch( Throwable tx )
        {
            LOGGER.error( "Error initializing EtherService.", tx );
            throw new ServletException( "Error initializing EtherService.", tx );
        }
    }

    /**
     * Creates an image with the plot and send it to the response
     * Call by a jsp with <img src="visualization/plotEther?....">
     *
     * @param request
     * @param response
     * @throws ServletException
     */
    public void doGet( final HttpServletRequest request, final HttpServletResponse response )
            throws ServletException
    {
        boolean flagException = false;

        try
        {
            final String title = request.getParameter( "title" );
            final String axeType = request.getParameter( "axeType" );

            // TODO : utiliser JSON pour récupérer pIdPIdList
            final String pIdPIdArrayString = request.getParameter( "pIdPIdList" );
            final List<Pair> pIdPIdList = extractpIdPIdListFromString( pIdPIdArrayString );
            if( null == pIdPIdList )
            {
                createErrorPane( response, request, WebException.WebCode.PLATEFORM_OR_PARAMETER_IS_NULL.toString(), null );
                flagException = true;
                throw new ServletException( WebException.WebCode.PLATEFORM_OR_PARAMETER_IS_NULL.toString(), null );
            }

            // Dates
            final String dateBegin = request.getParameter( "dateBegin" );
            final String dateEnd = request.getParameter( "dateEnd" );
            final Calendar calendar = Calendar.getInstance();
            Date formatedDateBegin = null;
            Date formatedDateEnd = null;
            if( null != dateBegin && null != dateEnd && !"false".equals( dateBegin ) && !"false".equals( dateEnd ) )
                try
                {
                    calendar.setTimeInMillis( Long.valueOf( dateBegin ) );
                    formatedDateBegin = calendar.getTime();
                    calendar.setTimeInMillis( Long.valueOf( dateEnd ) );
                    formatedDateEnd = calendar.getTime();
                }
                catch( Exception e )
                {
                    createErrorPane( response, request, WebException.WebCode.INVALID_DATE.toString(), e );
                    flagException = true;
                    throw new ServletException( WebException.WebCode.INVALID_DATE.toString(), e );
                }

            // Create plot
            final MegapoliPlot megapoliPlot = new MegapoliPlot();
            megapoliPlot.setTitle( title );
            megapoliPlot.setAxeType( axeType );
            megapoliPlot.setpIdPIdList( pIdPIdList );
            megapoliPlot.setBeginDate( formatedDateBegin );
            megapoliPlot.setEndDate( formatedDateEnd );

            if( AxeTypeForFixedPlateform.TIME_LINE.name().equals( axeType ) || AxeTypeForFixedPlateform.TIME_POINTS.name().equals( axeType ) )
            {
                megapoliPlot.setTimeSerie( true );
                megapoliPlot.setLegendToHide( pIdPIdList.size() == 1 );
                megapoliPlot.setValuesNumber( 10 );
            }
            else
            {
                // 2D plot : only one parameter to display
                final Integer plateformId = (Integer) pIdPIdList.get( 0 ).getFirstValue();
                final Integer parameterId = (Integer) pIdPIdList.get( 0 ).getSecondValue();
//                final Data valuesLists = _etherService.getListsByPlateformByParameterByPeriodFor2D( plateformId, parameterId, formatedDateBegin, formatedDateEnd );
                final Data valuesLists = BouchonHelper.createRealValuesFor2D();
                megapoliPlot.setData( valuesLists );

                megapoliPlot.setTimeSerie( false );
                megapoliPlot.setLegendToHide( false );
//                megapoliPlot.setValuesNumber( ( (ArrayList) valuesLists.getFirstArray() ).size() );
                megapoliPlot.setValuesNumber( 5 );
            }

            final BufferedImage bufferedImage = _etherPlotService.createMainPane( megapoliPlot, Context.getLocale( request ) );
            ImageIO.write( bufferedImage, "png", response.getOutputStream() );
        }
        catch( ServiceException se )
        {
            try
            {
                if( !flagException )
                    createErrorPane( response, request, se.getLocalizedMessage(), se );
            }
            catch( Exception e2 )
            {
                throw new ServletException( "Error : no possibity to write image in response", e2 );
            }
        }
        catch( Exception e1 )
        {
            try
            {
                if( !flagException )
                    createErrorPane( response, request, "", e1 );
            }
            catch( Exception e2 )
            {
                throw new ServletException( "Error : no possibity to write image in response", e2 );
            }
        }
    }

    // TODO : supprimer cette méthode et utiliser JSON pour récupérer pIdPIdList
    @Nullable
    private List<Pair> extractpIdPIdListFromString( @NotNull final String pIdPIdArrayString )
    {
        if( pIdPIdArrayString.isEmpty() )
            return null;

        final String[] pIDPIdArray = pIdPIdArrayString.split( "," );
        final List<Pair> pIdPIdList = new ArrayList<Pair>( pIDPIdArray.length );
        for( final String pIdPId : pIDPIdArray )
        {
            final String[] pIdPIdSplit = pIdPId.split( "-" );
            final Pair<Integer, Integer> pIdPIdPair = new Pair<Integer, Integer>();
            pIdPIdPair.setFirstValue( Integer.valueOf( pIdPIdSplit[0] ) );
            pIdPIdPair.setSecondValue( Integer.valueOf( pIdPIdSplit[1] ) );
            pIdPIdList.add( pIdPIdPair );
        }
        return pIdPIdList;
    }

    private void createErrorPane( final HttpServletResponse response, final HttpServletRequest request, @NotNull final String mainError, @Nullable final Exception e )
            throws IOException
    {
        final BufferedImage errorImage;
        if( null != e )
            errorImage = _etherPlotService.createErrorPane( mainError, e.toString(), Context.getLocale( request ) );
        else
            errorImage = _etherPlotService.createErrorPane( mainError, null, Context.getLocale( request ) );

        ImageIO.write( errorImage, "png", response.getOutputStream() );
    }

    private static final Log LOGGER = LogFactory.getLog( ControllerPlot.class );

    private EtherPlotService _etherPlotService;
    private EtherService _etherService;
}